/*!
 * @file GameEngine.cpp
 * @brief Q[GW̎
 * @date 2013/12/17
 * @author Deskull
 * 2014 Sikabane Works.
 */

#pragma once

#include "stdafx.h"
#include "GameEngine.h"

#include "HaveGameTime.h"
#include "Creature.h"
#include "GameCampaign.h"
#include "GameLogger.h"

namespace Deeangband
{
	GameEngine::GameEngine()
	{
		gameWorld = new GameWorld();
		gameSurface = new GameSurfaceSDL(gameWorld);
		gameLogger = new GameLogger();

		GameElement::GameWorldPtr = gameWorld;
		GameElement::GameSurfacePtr = gameSurface;
		GameElement::GameLoggerPtr = gameLogger;

		Dice::Initialize();
		gameWorld->Initialize();

		this->gameSurface->SetField(gameWorld->GetField(0));
		this->gameSurface->SetSideStatusCreature(gameWorld->GetCreature(0));
		this->gameLogger->PutLog("GameEngineJn");
		Direction::Initialize();
	}

	GameEngine::~GameEngine()
	{
		delete(gameLogger);
		delete(gameSurface);
		delete(gameWorld);
	}

	PLAY_EXIT_CODE GameEngine::PlayLoop(void)
	{
		GAME_COMMAND commandID;
		PLAY_EXIT_CODE code = PLAY_EXIT_NONE;
		ACTION_IT actionIt;
		gameSurface->Draw(gameWorld->GetPlayerCreature());

		while(code == PLAY_EXIT_NONE)
		{
			this->gameSurface->UpdateTimeStatus(this->gameWorld->GetGameTime());
			//actionIt = this->toNextProcess();
			commandID = gameSurface->GetCommand(gameWorld->GetPlayerCreature());
			code = DoGameCommand(commandID);
			this->gameSurface->FocusField(gameWorld->GetPlayerCreature()->second->GetPosition());

			this->gameSurface->UpdateSideCreatureStatus(gameWorld->GetPlayerCreature()->second.get());
			ProcessDeadCreatures();
			if(!gameWorld->GetPlayerCreature()->second->IsAlive())
			{
				code = PLAY_EXIT_DEAD;
				gameSurface->SystemMessage("Ȃ͎ɂ܂B");
			}
		}

		return PLAY_EXIT_QUIT;
	}

	PLAY_EXIT_CODE GameEngine::DoGameCommand(GAME_COMMAND command)
	{
		CREATURE_IT playerIt = gameWorld->GetPlayerCreature();

		switch(command)
		{
		case GAME_COMMAND_VIEW_PLAYER_STATUS:
			gameSurface->ViewCreatureStatus(gameWorld->GetPlayerCreature()->second.get());
			break;
		case GAME_COMMAND_REDRAW:
			gameSurface->Draw(playerIt);
			break;
		case GAME_COMMAND_DEBUG_XML_SAVE:
			gameWorld->XMLSave();
			gameSurface->SystemMessage("XMLɕۑ܂B");
			break;
		case GAME_COMMAND_DEBUG_XML_LOAD:
			gameSurface->Draw(playerIt);
			gameWorld->XMLLoad();
			gameSurface->SystemMessage("XMLĂяo܂B");
			break;
		case GAME_COMMAND_EXIT:
			return PLAY_EXIT_QUIT;
			break;
		case GAME_COMMAND_NORTH:
			playerIt->second.get()->Walk(DIRECTION_NORTH);
			break;
		case GAME_COMMAND_NORTH_EAST:
			playerIt->second.get()->Walk(DIRECTION_NORTH_EAST);
			break;
		case GAME_COMMAND_EAST:
			playerIt->second.get()->Walk(DIRECTION_EAST);
			break;
		case GAME_COMMAND_SOUTH_EAST:
			playerIt->second.get()->Walk(DIRECTION_SOUTH_EAST);
			break;
		case GAME_COMMAND_SOUTH:
			playerIt->second.get()->Walk(DIRECTION_SOUTH);
			break;
		case GAME_COMMAND_SOUTH_WEST:
			playerIt->second.get()->Walk(DIRECTION_SOUTH_WEST);
			break;
		case GAME_COMMAND_WEST:
			playerIt->second.get()->Walk(DIRECTION_WEST);
			break;
		case GAME_COMMAND_NORTH_WEST:
			playerIt->second.get()->Walk(DIRECTION_NORTH_WEST);
			break;
		}
		return PLAY_EXIT_NONE;
	}


	bool GameEngine::CombatCreatures(Creature *attackerPtr, Creature *targetPtr)
	{
		attackerPtr->Melee(targetPtr);
		return true;
	}

	int GameEngine::ProcessDeadCreatures()
	{
		int deadNum = 0;
		CREATURE_IT creatureIt, creatureItCurrent;
		CREATURE_LIST *creatureList = gameWorld->GetCreatureList();

		creatureIt = creatureList->begin();

		while(creatureIt != creatureList->end())
		{
			creatureItCurrent = creatureIt;
			creatureIt++;
			if(!creatureItCurrent->second->IsAlive())
			{
				//!< @note vC[̎SȂΕʓr̂ŃXLbvAȂ΃N[`[̏ŁB
				if(gameWorld->GetPlayerCreature() != creatureItCurrent)
				{
					gameSurface->GameMessage(creatureItCurrent->second->GetName() + "͎񂾁B");
					creatureList->erase(creatureItCurrent);
				}
				deadNum++;
			}
		}

		return deadNum;
	}

	ACTION_IT GameEngine::toNextProcess(void)
	{
		return this->gameWorld->GetActionList()->begin()++;
	}

}
